@@ -1,14 +1,15 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import hashlib
+
+from decimal import Decimal
 from itertools import groupby
 
 from trytond.i18n import gettext
 from trytond.model import ModelView, Workflow, fields
-from trytond.pool import PoolMeta, Pool
-from trytond.pyson import Eval, Bool, If
-
 from trytond.modules.product import round_price
+from trytond.pool import Pool, PoolMeta
+from trytond.pyson import Bool, Eval, If
+from trytond.transaction import Transaction
 
 from .exceptions import PurchaseWarning
 
@@ -30,7 +31,7 @@ class Routing(metaclass=PoolMeta):
             'required': Bool(Eval('supplier')),
             'invisible': ~Eval('supplier'),
             },
-        depends=['supplier', 'supplier_service_supplier'],
+        depends={'supplier_service_supplier'},
         help="The service to buy to the supplier for the production.")
     supplier_service_supplier = fields.Many2One(
         'purchase.product_supplier', "Supplier's Service",
@@ -51,14 +52,12 @@ class Routing(metaclass=PoolMeta):
         states={
             'invisible': ~Eval('supplier'),
             },
-        depends=['supplier_service', 'supplier'],
         help="The supplier's service to buy for the production.")
     supplier_quantity = fields.Float("Quantity",
         states={
             'invisible': ~Eval('supplier_service'),
             'required': Bool(Eval('supplier_service')),
             },
-        depends=['supplier_service'],
         help="The quantity to buy to produce one time the BOM.")
 
     @classmethod
@@ -112,7 +111,6 @@ class Production(metaclass=PoolMeta):
         domain=[
             ('purchase.company', '=', Eval('company', -1)),
             ],
-        depends=['company'],
         help="The lines to add to the production cost.")
 
     def get_cost(self, name):
@@ -159,7 +157,6 @@ class Production(metaclass=PoolMeta):
 
         super(Production, cls).wait(productions)
 
-        today = Date.today()
         purchases = []
         lines = []
 
@@ -170,6 +167,9 @@ class Production(metaclass=PoolMeta):
                 key=cls._group_purchase_key))
         for key, grouped in groupby(productions, key=cls._group_purchase_key):
             productions = list(grouped)
+            key = dict(key)
+            with Transaction().set_context(company=key['company'].id):
+                today = Date.today()
             try:
                 purchase_date = min(p.planned_start_date or p.planned_date
                     for p in productions if p.planned_date)
@@ -178,7 +178,7 @@ class Production(metaclass=PoolMeta):
             if purchase_date < today:
                 purchase_date = today
             purchase = Purchase(purchase_date=purchase_date)
-            for f, v in key:
+            for f, v in key.items():
                 setattr(purchase, f, v)
             purchases.append(purchase)
             for production in productions:
@@ -212,11 +212,13 @@ class Production(metaclass=PoolMeta):
         line.product_supplier = self.routing.supplier_service_supplier
         line.unit = self.routing.supplier_service.purchase_uom
         factor = self.bom.compute_factor(
-            self.product, self.quantity or 0, self.uom)
+            self.product, self.quantity or 0, self.unit)
         line.quantity = line.unit.round(
             factor * self.routing.supplier_quantity)
         line.purchase = purchase
         line.on_change_product()
+        if line.unit_price is None:
+            line.unit_price = round_price(Decimal(0))
         return line
 
     @classmethod
@@ -234,8 +236,7 @@ class Production(metaclass=PoolMeta):
             names = ', '.join(p.rec_name for p in productions[:5])
             if len(pendings) > 5:
                 names += '...'
-            warning_name = '%s.pending_purchase.done' % hashlib.md5(
-                str(pendings).encode('utf-8')).hexdigest()
+            warning_name = Warning.format('pending_purchase.done', pendings)
             if Warning.check(warning_name):
                 raise PurchaseWarning(
                     warning_name,
